"""
$RCSfile: CreationDialog.py,v $

Implements the GUI mini-workflow invoked from the Zope Management
Interface to create an instance of ExternalFile.  This source code has
been factored out of ExternalFile into this separate module for the
reasons listed below:

1. This code is not strictly necessary to use ExternalFile, and in
certain cases (for example, when creating instances programmatically)
one prefers to bypass this code altogether and call the ExternalFile
constructor directly.

2. There are a great many possible error conditions or alternative
scenarios that may occur when creating an instance of ExternalFile TTW
(through the web), resulting in many lines of code.  This code was
threatening to obscure the actual implementation code when it was all
contained in the same module.

3. Zope3 introduces an architecture where GUI handling code and Zope
object implementation code are kept separate and knit together
declaratively through the use of a deployment descriptor.  This allows
alternative GUIs to be plugged in and makes the code much more
general.  For example, it could be used in non-Zope environments.
Creating a separate module for GUI stuff moves us in that direction.

Author: <a href="mailto:cstrong@arielpartners.com">Craeg Strong</a>
Release: 1.0
"""

__cvstag__  = '$Name:  $'[6:-2]
__date__    = '$Date: 2002/05/26 03:36:17 $'[6:-2]
__version__ = '$Revision: 1.3 $'[10:-2]

#
# Local modules
#
from FileUtils import copy_file, create_file

# Zope builtins
from Globals import DTMLFile, MessageDialog

# Python builtins
from cStringIO import StringIO
import os, socket, string, tempfile

# globals() is standard python for passing the current namespace
manage_addForm                = DTMLFile('dtml/create', globals())
create_externalfile_formpart  = DTMLFile('dtml/create_externalfile_formpart', globals())
create_standard_formpart      = DTMLFile('dtml/create_standard_formpart', globals())
confirm_create_form           = DTMLFile('dtml/confirm_create_form', globals())
conflict_create_form          = DTMLFile('dtml/conflict_create_form', globals())

def manage_add_via_gui(self,
                       id='', title='', description='',                   # standard
                       target_filepath='', basedir='', upload_file=None,  # externalfile
                       REQUEST=None):
    """
    
    Workhorse method called from create.dtml form.  Actually does the
    work of confirming the user's intended (possibly destructive)
    actions, creating the target file if necessary, and calling
    create_object to actually create the object.  This method may be
    called from create callbacks of derived classes to do the heavy
    lifting.  Derived classes that call this method must define a
    manage_add method (see ExternalFile.manage_add) to actually create
    the proper instance.

    """
    success_url  = 'manage_main'
    tryagain_url = 'create'
    
    if not target_filepath:
        return MessageDialog(title   = 'ERROR',
                             message = 'You must specify a target filepath.',
                             action  = tryagain_url)
    if not id:
        id = target_filepath
        id = id[max(string.rfind(id,'/'), 
                    string.rfind(id,'\\'), 
                    string.rfind(id,':')
                    )+1:]

    # Six possible scenarios:
    #
    # [One]   No upload, target filepath already exists on the server and is readable.
    #         Action: Point to it.
    #
    # [Two]   No upload, target filepath does not exist on the server.
    #         Action: Give option to create blank file or cancel.
    #
    # [Three] No upload, target filepath exists on the server but is not readable.
    #         Action: Display error message, allow user to try again.
    #
    # [Four]  Upload, target filepath does not exist on the server.
    #         Action: Give option to create file with uploaded contents or cancel.
    #
    # [Five]  Upload, target filepath exists on the server but is not writeable.
    #         Action: Display error message, allow user to try again.
    #
    # [Six]   Upload, target filepath exists on the server and is writable.
    #         Action: Give options: overwrite/dont overwrite/try again
    #
    # [Seven] Upload, upload file is empty
    #         Action: Display error message, allow user to try again
    #
    fully_resolved_target_filepath = os.path.join(basedir,target_filepath)
    
    if not upload_file or not upload_file.filename:
        if not os.path.exists(fully_resolved_target_filepath):
            # Scenario Two
            message= """File %s does not exist yet on server %s.
            Would you like to create a blank file by that name?\n""" % ( fully_resolved_target_filepath,
                                                                         socket.gethostname() )
            return self.confirm_create_form(message          = message,
                                            file_id          = id, # id -> file_id or acquisition will get wrong one
                                            file_title       = title,
                                            file_description = description,
                                            target_filepath  = target_filepath,
                                            basedir          = basedir,
                                            temp_filename    = '',
                                            upload_filename  = '')
        else:
            try:
                testopen = open(fully_resolved_target_filepath, 'rb')
            except IOError, value:
                # Scenario Three
                message = """Target file %s exists on server %s
                but is not readable (%s).\n""" % (
                    fully_resolved_target_filepath, socket.gethostname(), value )
                message.replace('\n','<br/>')
                return MessageDialog(title   = 'ERROR',
                                     message = message,
                                     action  = tryagain_url)
            else:
                # Scenario One
                self.manage_add(id, title, description,
                                target_filepath, basedir, REQUEST)
                message = """Object instance \'%s\' successfully created,
                pointing to file %s on %s.\n""" % (
                    id, fully_resolved_target_filepath, socket.gethostname() )
                message.replace('\n','<br/>')                
                return MessageDialog(title   = 'CREATED',
                                     message = message,
                                     action  = success_url)
    else:
        #
        # we have an uploaded file.  Grab the contents and store in a
        # temporary file, copy it over to target_pathname iff the user
        # confirms this potentially destructive action
        #
        temp_filename = tempfile.mktemp()
        upload_contents = upload_file.read()
        if not upload_contents:
            message = """Uploaded file %s is empty.
            Please specify a non-empty file.\n""" % upload_file.filename
            message.replace('\n','<br/>')
            return MessageDialog(title   = 'ERROR',
                                 message = message,
                                 action  = tryagain_url)
        else:
            data = StringIO(upload_contents)
            copy_file(data, temp_filename)
            if not os.path.exists(fully_resolved_target_filepath):
                # Scenario Four
                message = """File %s does not exist yet on server %s.
                Would you like to create a file by that name containing the uploaded contents?
                """ % ( fully_resolved_target_filepath, socket.gethostname() )
                return self.confirm_create_form(message          = message,
                                                file_id          = id, # id -> file_id or acquisition will get wrong one
                                                file_title       = title,
                                                file_description = description,
                                                target_filepath  = target_filepath,
                                                basedir          = basedir,
                                                temp_filename    = temp_filename,
                                                upload_filename  = upload_file.filename)
            else:
                try:
                    testopen = open(fully_resolved_target_filepath, 'ab')
                    testopen.close()
                except IOError, value:
                    # Scenario Five
                    message = """Target file %s is not writable
                    on server %s (%s).\n""" % (
                        fully_resolved_target_filepath, socket.gethostname(), value )
                    message.replace('\n','<br/>')                    
                    return MessageDialog(title   = 'ERROR',
                                         message = message,
                                         action  = tryagain_url)
                else:
                    # Scenario Six
                    return self.conflict_create_form(file_id          = id, # change id -> file_id else conflict
                                                     file_title       = title,
                                                     file_description = description,
                                                     target_filepath  = target_filepath,
                                                     basedir          = basedir,
                                                     temp_filename    = temp_filename,
                                                     upload_filename  = upload_file.filename)



def confirm_create_action(self,
                          file_id, file_title, file_description,
                          target_filepath, basedir,
                          temp_filename, upload_filename,
                          REQUEST=None):
    """
    
    Create a new instance of ExternalFile pointing to target filepath
    on server.  If user has uploaded a file (upload_file is not None),
    overwrite the contents of the server file with the uploaded file
    contents.
    
    """
    fully_resolved_target_filepath = os.path.join(basedir,target_filepath)
    target_is_existing_file = os.path.exists(fully_resolved_target_filepath)
    
    create_msg = ""
    if upload_filename:
        try:
            create_file(temp_filename, fully_resolved_target_filepath)
        except IOError, value:
            message = """Target file %s is not writable
            on server %s (%s).\n""" % (
                fully_resolved_target_filepath, socket.gethostname(), value )
            message.replace('\n','<br/>')
            return MessageDialog(title   = 'ERROR',
                                 message = message,
                                 action  = 'create')
        if target_is_existing_file:
            message = """Object instance %s successfully created,
            pointing to file %s on %s.
            Contents of %s:%s
            were replaced by contents of uploaded file %s.
            File successfully created.\n""" % (
                file_id, fully_resolved_target_filepath, socket.gethostname(),
                socket.gethostname(), fully_resolved_target_filepath,
                upload_filename)
        else:
            message = """Object instance %s successfully created,
            pointing to newly-created file %s on %s,
            with contents copied from uploaded file %s.
            File successfully created.\n""" % (
                file_id, fully_resolved_target_filepath, socket.gethostname(),
                upload_filename )
    else:
        if not target_is_existing_file:
            try:
                create_file(None, fully_resolved_target_filepath)
            except IOError, value:
                message = """Target file %s is not writable
                on server %s (%s).\n""" % (
                    fully_resolved_target_filepath, socket.gethostname(), value)
                message.replace('\n','<br/>')
                return MessageDialog(title   = 'ERROR',
                                     message = message,
                                     action  = 'create')
        message = """Object instance %s successfully created,
        pointing to file %s on %s.
        File successfully created.\n""" % (
            file_id, fully_resolved_target_filepath, socket.gethostname() )

    self.manage_add(file_id, file_title, file_description,
                    target_filepath, basedir, REQUEST)

    message.replace('\n','<br/>')
    return MessageDialog(title   = 'CREATED',
                         message = message,
                         action  = 'manage_main')
